<?php


namespace App\Console\Commands;


use App\Models\Article;
use App\Models\ArticleContent;
use App\Models\PortalUrl;
use App\Services\ArticleImporterInterface;
use Illuminate\Console\Command;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Support\Facades\DB;

class ImportArticleContent extends Command
{
    /**
     * The console command name.
     *
     * @var string
     */
    protected $signature = "import:article-content";

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = "Import article content";

    /** @var array  */
    private $importServices = [];



    public function handle()
    {
        try {
            /** @var Collection $articles */
            $articles = Article::where('cannot_be_found_404', 0)
                ->where('redirected_301', 0)
                ->where('unable_parse_content', 0)
                ->get();

            $this->info("Number of articles selected:".$articles->count());

            /** @var Article $article */
            foreach ($articles as $article) {
                /** @var ArticleContent| $content */
                $content = ArticleContent::where('article_id', $article->id)->first();
                if(!empty($content)) {
                    $this->info("Article with ID:".$article->id." was already imported, continue.");
                    continue;
                }


                $this->info("Start importing article ID:".$article->id);
                /** @var PortalUrl $portalUrlImportService */
                $portalUrl = $article->portalUrl()->first();
                $urlImportService = $portalUrl->article_importer_service;
                if(isset($this->importServices[$urlImportService])) {
                    $this->info("Using existing service");
                    $importService = $this->importServices[$urlImportService];
                } else {
                    $this->info("Using new service");
                    $importService = new $urlImportService();
                    $this->importServices[$urlImportService] = $importService;
                }
                if($importService instanceof ArticleImporterInterface) {
                    if(strpos($article->link, 'http') === false) {
                        /** @var PortalUrl $portaUrlTmp */
                        $portaUrlTmp = $article->portalUrl()->first();
                        $portaUrlTmp->deleting_content = 1;
                        $portaUrlTmp->save();
                        $article->delete();
                        continue;
                    }
                    $exists = true;
                    $errorMessage = '';
                    $file_headers = @get_headers($article->link);
                    if(!empty($file_headers)) {
                        foreach ($file_headers as $key => $value) {
                            if($value == 'HTTP/1.1 404 Not Found' || $value == 'HTTP/1.0 404 Not Found' || $value == 'HTTP/1.1 404 File Not Found') {
                                $exists = false;
                                /** @var PortalUrl $portaUrlTmp */
                                $portaUrlTmp = $article->portalUrl()->first();
                                $portaUrlTmp->deleting_content = 1;
                                $portaUrlTmp->save();

                                $article->cannot_be_found_404 = 1;
                                $article->save();
                                $errorMessage = $value;
                                break;
                            } elseif ($value == 'HTTP/1.0 400 Bad request' && strpos($article->link, ' ') !== false) {
                                $link = str_replace(' ', '%20', $article->link);
                                $article->link = $link;
                                $article->incorrectly_formated_400 = 1;
                                $article->save();
                                $exists = true;
                            } elseif ($value == 'HTTP/1.1 301 Moved Permanently' && strpos($article->link, ' ') !== false) {
                                $exists = false;
                                /** @var PortalUrl $portaUrlTmp */
                                $portaUrlTmp = $article->portalUrl()->first();
                                $portaUrlTmp->deleting_content = 1;
                                $portaUrlTmp->save();

                                $article->redirected_301 = 1;
                                $article->save();
                                $errorMessage = $value;
                            } elseif ($value == 'HTTP/1.1 301 Moved Permanently') {
                                /** @var PortalUrl $portaUrlTmp */
                                $portaUrlTmp = $article->portalUrl()->first();
                                $portaUrlTmp->deleting_content = 1;
                                $portaUrlTmp->save();
                                $exists = false;

                                $article->redirected_301 = 1;
                                $article->save();
                                $errorMessage = $value;
                            }
                        }
                    } else {
                        $exists = true;
                    }

                    if($exists) {
                        $articleContent = $importService->getArticleContent($article->link);
                        if(empty($articleContent['htmlContent']) && empty($articleContent['content'])) {
                            $article->unable_parse_content = 1;
                            $article->save();
                            $this->info("Article imported unsuccessfully. Moving to unsuccessfully stack");
                        } else {
                            $newArticleContent = new ArticleContent();
                            $newArticleContent->html_content = $articleContent['htmlContent'];
                            $newArticleContent->content = $articleContent['content'];
                            $newArticleContent->article_id = $article->id;
                            $newArticleContent->save();
                            $this->info("Article imported successfully");
                        }
                    } else {
                        $this->info("Article is no longer active on URL ".$article->link." . Removing article from active stack: ".$errorMessage);
                    }
                } else {
                    $this->info("Article Importer is not a valid service");
                }
            }

        } catch (\Exception $e) {
            $this->error($e->getMessage());
        }
    }
}