<?php

namespace App\Console\Commands;

use App\Models\Article;
use App\Models\Portal;
use App\Models\PortalUrl;
use Illuminate\Console\Command;
use Illuminate\Database\Eloquent\Collection;

class ImportFullContent extends Command
{
    /**
     * The console command name.
     *
     * @var string
     */
    protected $signature = "import:full";

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = "Import everything";



    public function handle()
    {
        try {
            $this->info("Start import");
            /** @var Collection $allPortals */
            $allPortals = Portal::all();
            /** @var Portal $portal */
            foreach ($allPortals as $portal) {
                $this->info("Importing portal ".$portal->title);
                /** @var Collection $allUrlsForPortal */
                $allUrlsForPortal = $portal->urls()->where('active', '=', 1)->get();
                /** @var PortalUrl $url */
                foreach ($allUrlsForPortal as $url) {
                    $this->info("Importing portal url ".$url->url);
                    try {
                        $file = file_get_contents($url->url);
                    } catch (\Exception $exception) {
                        $url->allow_crawling = 0;
                        $url->save();
                        $agent = 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.0.3705; .NET CLR 1.1.4322)';

                        $ch = curl_init();
                        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
                        curl_setopt($ch, CURLOPT_VERBOSE, true);
                        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                        curl_setopt($ch, CURLOPT_USERAGENT, $agent);
                        curl_setopt($ch, CURLOPT_URL,$url->url);
                        $file = curl_exec($ch);
                        curl_close($ch);
                    }
                    try {
                        $xmlFile = simplexml_load_string($file);
                    } catch (\Exception $exception) {
                        continue;
                    }

                    foreach ($xmlFile->channel->item as $article) {
                        $isArticleIn = Article::where('link', '=', $article->link)->first();
                        if($isArticleIn instanceof Article) {
                            $this->info("Existing articel with URL ".$article->link);
                            continue;
                        } else {
                            try {
                                $this->info("Importing articel with URL ".$article->link);
                                $newArticle = new Article();
                                $newArticle->title = trim($article->title);
                                $newArticle->link = trim($article->link);
                                $newArticle->description = trim($article->description);
                                $newArticle->portal_url_id = $url->id;
                                $newArticle->allow_crawling = $url->allow_crawling;
                                $newArticle->save();
                                $this->info("Articel imported");
                            } catch (\Exception $exception) {
                                $this->info("Articel cannot be imported. Continue...");
                            }
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            $this->error($e->getMessage());
        }
    }
}